@@ -19,12 +19,11 @@ class ProjectPermission(BasePermission):
 
     def has_object_permission(self, request, view, obj):
         if request.method == 'DELETE':
-            return (request.user.has_perm('projects.delete_project') or
-                    request.user.has_perm('projects.delete_project', obj))
+            permission = 'projects.delete_project'
+        else:
+            permission = 'projects.change_project'
 
-        if request.method in ['PATCH', 'PUT']:
-            return (request.user.has_perm('projects.change_project') or
-                    request.user.has_perm('projects.change_project', obj))
+        return (request.user.has_perm(permission) or request.user.has_perm(permission, obj))
 
 
 class ReadOnly(BasePermission):
