@@ -3,6 +3,8 @@ import logging
 from django.shortcuts import get_object_or_404
 from rest_framework import mixins, viewsets
 from rest_framework.permissions import DjangoModelPermissions, IsAuthenticated, IsAdminUser
+from reversion import set_comment
+from reversion.views import RevisionMixin
 
 from huscy.projects import serializer, services
 from huscy.projects.models import Project
@@ -20,8 +22,8 @@ def get_client_ip(request):
     return ip
 
 
-class MembershipViewSet(mixins.CreateModelMixin, mixins.DestroyModelMixin, mixins.ListModelMixin,
-                        mixins.UpdateModelMixin, viewsets.GenericViewSet):
+class MembershipViewSet(RevisionMixin, mixins.CreateModelMixin, mixins.DestroyModelMixin,
+                        mixins.ListModelMixin, mixins.UpdateModelMixin, viewsets.GenericViewSet):
     serializer_class = serializer.MembershipSerializer
     permission_classes = (IsAuthenticated, IsAdminUser | IsProjectCoordinator | ReadOnly)
 
@@ -34,25 +36,31 @@ class MembershipViewSet(mixins.CreateModelMixin, mixins.DestroyModelMixin, mixin
 
     def perform_create(self, serializer):
         membership = serializer.save(project=self.project)
+        set_comment('Created membership <ID-{membership.id}> for project <ID-{self.project.id}>')
+
         logger.info('Membership created by user %s from IP %s for project "%s" and member %s',
                     self.request.user.username, get_client_ip(self.request),
                     membership.project.title, membership.user.get_full_name())
 
     def perform_destroy(self, membership):
         services.delete_membership(membership)
+        set_comment('Deleted membership <ID-{membership.id}>')
+
         logger.info('Membership deleted by user %s from IP %s for project "%s" and member %s',
                     self.request.user.username, get_client_ip(self.request),
                     membership.project.title, membership.user.get_full_name())
 
     def perform_update(self, serializer):
         membership = serializer.save()
+        set_comment('Updated membership <ID-{membership.id}>')
+
         logger.info('Membership updated by user %s from IP %s for project "%s" and member %s',
                     self.request.user.username, get_client_ip(self.request),
                     membership.project.title, membership.user.get_full_name())
 
 
-class ProjectViewSet(mixins.CreateModelMixin, mixins.DestroyModelMixin, mixins.ListModelMixin,
-                     mixins.UpdateModelMixin, viewsets.GenericViewSet):
+class ProjectViewSet(RevisionMixin, mixins.CreateModelMixin, mixins.DestroyModelMixin,
+                     mixins.ListModelMixin, mixins.UpdateModelMixin, viewsets.GenericViewSet):
     serializer_class = serializer.ProjectSerializer
     permission_classes = (IsAuthenticated, ProjectPermission)
 
@@ -61,21 +69,39 @@ class ProjectViewSet(mixins.CreateModelMixin, mixins.DestroyModelMixin, mixins.L
 
     def perform_create(self, serializer):
         project = serializer.save()
+        set_comment(f'Created project <ID-{project.id}>')
+
         logger.info('Project created by user %s from IP %s with title "%s"',
                     self.request.user.username, get_client_ip(self.request), project.title)
 
     def perform_destroy(self, project):
         services.delete_project(project)
+        set_comment(f'Deleted project <ID-{project.id}>')
+
         logger.info('Project deleted by user %s from IP %s with title "%s"',
                     self.request.user.username, get_client_ip(self.request), project.title)
 
     def perform_update(self, serializer):
         project = serializer.save()
+        set_comment(f'Updated project <ID-{project.id}>')
+
         logger.info('Project updated by user %s from IP %s with title "%s"',
                     self.request.user.username, get_client_ip(self.request), project.title)
 
 
-class ResearchUnitViewSet(viewsets.ModelViewSet):
+class ResearchUnitViewSet(RevisionMixin, viewsets.ModelViewSet):
     permission_classes = (DjangoModelPermissions, )
     queryset = services.get_research_units()
     serializer_class = serializer.ResearchUnitSerializer
+
+    def perform_create(self, serializer):
+        research_unit = serializer.save()
+        set_comment(f'Created research unit <ID-{research_unit.id}>')
+
+    def perform_destroy(self, research_unit):
+        research_unit.delete()
+        set_comment(f'Deleted research unit <ID-{research_unit.id}>')
+
+    def perform_update(self, serializer):
+        research_unit = serializer.save()
+        set_comment(f'Updated research unit <ID-{research_unit.id}>')
